==> lmtpd-0.9.19.tar.gz: lmtpd-0.9.19/quotadb_mysql.c <==
/*
   Copyright 2000, 2001, 2002, 2003 Laurent Wacrenier

   This file is part of PLL lmtpd.

   PLL lmtpd is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   PLL lmtpd is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with PLL lmtpd; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "config.h"

#if WITH_MYSQLLOG

static char const rcsid[] =
"$Id: quotadb_mysql.c,v 1.8 2003/03/28 14:41:01 lwa Exp $";

#include <mysql.h>
#include <stdarg.h>
#include <syslog.h>
#include <stdlib.h>
#include <string.h>

#include "lmtp.h"

#define QUERY_LEN 1000

typedef char quser_t[QUERY_LEN*2+1];

static int quotadb_mysql_open_server(struct fstate *state) {
  if (state->quota_db_mysql_opened && mysql_ping(&state->quota_db_mysql)!=0) {
    syslog(LOG_ERR, "MySQL Quota ping: %s", mysql_error(&state->quota_db_mysql));
    mysql_close(&state->quota_db_mysql);
    state->quota_db_mysql_opened=0;
  }
  if (state->quota_db_mysql_opened)
    return 0;
  mysql_init(&state->quota_db_mysql);
  if (mysql_real_connect(&state->quota_db_mysql,
                         state->quota_db_mysql_host,
                         state->quota_db_mysql_user,
                         state->quota_db_mysql_password,
                         state->quota_db_mysql_database,
                         0, NULL, 0)) {
    state->quota_db_mysql_opened=1;
    return 0;
  }
  syslog(LOG_ERR, "MySQL Quota open: %s", mysql_error(&state->quota_db_mysql));
  return 0;
}

static int quote_user(struct fstate *state, char *user, quser_t quser) {
  int user_len = strlen(user);
  if (user_len > QUERY_LEN)
    return -1;
  return mysql_real_escape_string(&state->quota_db_mysql, 
				  quser, user, user_len);
}

static int query(struct fstate *state, char *fmt, ...) {
  char buffer[QUERY_LEN];
  int len;
  int ret;
  va_list ap;

  va_start(ap, fmt);
  len = vsnprintf(buffer, QUERY_LEN, fmt, ap);
  va_end(ap);
  if (len > QUERY_LEN)
    return -1;
  ret = mysql_real_query(&state->quota_db_mysql, buffer, len);
  if (ret != 0) {
    syslog(LOG_NOTICE, "quotadb_mysql.c: error in query: %s", 
	   mysql_error(&state->quota_db_mysql));
    return -1;
  }
  return 0;
}

/*
 *  quotadb_mysql_* functions
 */

int quotadb_mysql_open(struct fstate *state) {
  if (state->quota_db_mysql_col_time == NULL ||
      state->quota_db_mysql_col_code == NULL ||
      state->quota_db_mysql_col_num == NULL  ||
      state->quota_db_mysql_table == NULL ||
      state->quota_db_mysql_col_user == NULL) {
    syslog(LOG_ERR, "quotadb_mysql: NULL column or table name");
    return -1;
  }
  return quotadb_mysql_open_server(state);
}



void quotadb_mysql_close(struct fstate *state) {
  if (state->quota_db_mysql_opened) {
    mysql_close(&state->quota_db_mysql);
    state->quota_db_mysql_opened = 0;
  }
}


void quotadb_mysql_del(struct fstate *state, char *user) {
  quser_t quser;
  if (quote_user(state, user, quser) == -1)
    return;
  if (query(state, "delete from `%s` where `%s` = '%s'",
	    state->quota_db_mysql_table,
	    state->quota_db_mysql_col_user,
	    quser) == -1)
    return;
}

char *quotadb_mysql_check(struct fstate *state, char *user) {
  quser_t quser;
  MYSQL_RES *res;
  MYSQL_ROW row;
  int date = 0;
  int num = 0;
  int code = 0;
  if (quote_user(state, user, quser) == -1)
    return NULL;
  if (query(state,
	    "select `%s`, `%s`, `%s` from `%s` where `%s` = '%s' limit 1",
	    state->quota_db_mysql_col_time,
	    state->quota_db_mysql_col_code,
	    state->quota_db_mysql_col_num,
	    state->quota_db_mysql_table,
	    state->quota_db_mysql_col_user,
	    quser) == -1)
    return NULL;
  if ( (res = mysql_use_result(&state->quota_db_mysql)) == NULL )
    return NULL;
  if ( (row = mysql_fetch_row(res)) == NULL ) {
    mysql_free_result(res);
    return NULL;
  }
  if (row[0]) date = strtoul(row[0], NULL, 10);
  if (row[1]) code = strtol(row[1], NULL, 10);
  if (row[2]) num  = strtol(row[2], NULL, 10);

  mysql_free_result(res);

  if (state->date - date < state->quota_db_timeout) {
    num ++;
    if (state->quota_maxchecks > 0 && num > state->quota_maxchecks)
      code = 552;
    query(state,
	  "update low_priority `%s`"
	  " set `%s` = %ld, `%s` = %d,  `%s` = `%s` + 1",
	  state->quota_db_mysql_table,
	  state->quota_db_mysql_col_time,
	  (unsigned long)state->date,
	  state->quota_db_mysql_col_code,
	  code,
	  state->quota_db_mysql_col_num,
	  state->quota_db_mysql_col_num
	  );
    
    return seterr(state, code, "4.2.2",
		  "<%s>: Mailbox full -- Quota exceeded", user);
    
  } else {
    query(state, "delete from `%s` where `%s` = '%s'",
	  state->quota_db_mysql_table,
	  state->quota_db_mysql_col_user,
	  quser);
  }
  return NULL;
}

void quotadb_mysql_add(struct fstate *state, char *user, int code) {
  quser_t quser;
  if (quote_user(state, user, quser) == -1)
    return;
  if (query(state, 
	    "REPLACE LOW_PRIORITY `%s` (`%s`,`%s`,`%s`,`%s`)"
	    " VALUES ('%s',%d,%d,%lu)",
	    state->quota_db_mysql_table,
	    state->quota_db_mysql_col_user,
	    state->quota_db_mysql_col_code,
	    state->quota_db_mysql_col_num,
	    state->quota_db_mysql_col_time,
	    quser, code, 1, (unsigned long)state->date) == -1)
    return;
  return;
}

/*
 *  quotadb_mysqladd_* functions
 */

int quotadb_mysqladd_open(struct fstate *state) {
  if (state->quota_db_mysql_col_user == NULL ||
      state->quota_db_mysql_table == NULL) {
    syslog(LOG_ERR, "quotadb_mysqladd: NULL user column or table name");
    return -1;
  }
  return quotadb_mysql_open_server(state);
}

void quotadb_mysqladd_add(struct fstate *state, char *user, int code) {
  quser_t quser;
  if (quote_user(state, user, quser) == -1)
    return;
  query(state,
	"REPLACE LOW_PRIORITY `%s` SET `%s`='%s'",
	state->quota_db_mysql_table,
	state->quota_db_mysql_col_user,
	quser);
  return;
}

#endif

==> lmtpd-0.9.24.tar.gz: lmtpd-0.9.24/quotadb_mysql.c <==
/*
   Copyright 2000, 2001, 2002, 2003 Laurent Wacrenier

   This file is part of PLL lmtpd.

   PLL lmtpd is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   PLL lmtpd is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with PLL lmtpd; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/




#include "config.h"

#if WITH_MYSQLLOG

static char const rcsid[] UNUSED =
"$Id: quotadb_mysql.c,v 1.11 2005/01/26 10:31:14 lwa Exp $";

#include <mysql.h>
#include <stdarg.h>
#include <syslog.h>
#include <stdlib.h>
#include <string.h>

#include "lmtp.h"

#define QUERY_LEN 1000

typedef char quser_t[QUERY_LEN*2+1];

static int quotadb_mysql_open_server(struct fstate *state) {
  if (state->quota_db_mysql_opened && mysql_ping(&state->quota_db_mysql)!=0) {
    syslog(LOG_ERR, "MySQL Quota ping: %s", mysql_error(&state->quota_db_mysql));
    mysql_close(&state->quota_db_mysql);
    state->quota_db_mysql_opened=0;
  }
  if (state->quota_db_mysql_opened)
    return 0;
  mysql_init(&state->quota_db_mysql);
  if (mysql_real_connect(&state->quota_db_mysql,
                         state->quota_db_mysql_host,
                         state->quota_db_mysql_user,
                         state->quota_db_mysql_password,
                         state->quota_db_mysql_database,
                         0, NULL, 0)) {
    state->quota_db_mysql_opened=1;
    return 0;
  }
  syslog(LOG_ERR, "MySQL Quota open: %s", mysql_error(&state->quota_db_mysql));
  return 0;
}

static int quote_user(struct fstate *state, char *user, quser_t quser) {
  int user_len = strlen(user);
  if (user_len > QUERY_LEN)
    return -1;
  return mysql_real_escape_string(&state->quota_db_mysql, 
				  quser, user, user_len);
}

static int query(struct fstate *state, char *fmt, ...) {
  char buffer[QUERY_LEN];
  int len;
  int ret;
  va_list ap;

  va_start(ap, fmt);
  len = vsnprintf(buffer, QUERY_LEN, fmt, ap);
  va_end(ap);
  if (len > QUERY_LEN)
    return -1;
  ret = mysql_real_query(&state->quota_db_mysql, buffer, len);
  if (ret != 0) {
    syslog(LOG_NOTICE, "quotadb_mysql.c: error in query: %s", 
	   mysql_error(&state->quota_db_mysql));
    return -1;
  }
  return 0;
}

/*
 *  quotadb_mysql_* functions
 */

int quotadb_mysql_open(struct fstate *state) {
  if (state->quota_db_mysql_col_time == NULL ||
      state->quota_db_mysql_col_code == NULL ||
      state->quota_db_mysql_col_num == NULL  ||
      state->quota_db_mysql_table == NULL ||
      state->quota_db_mysql_col_user == NULL) {
    syslog(LOG_ERR, "quotadb_mysql: NULL column or table name");
    return -1;
  }
  return quotadb_mysql_open_server(state);
}



void quotadb_mysql_close(struct fstate *state) {
  if (state->quota_db_mysql_opened) {
    mysql_close(&state->quota_db_mysql);
    state->quota_db_mysql_opened = 0;
  }
}


void quotadb_mysql_del(struct fstate *state, char *user) {
  quser_t quser;
  if (quote_user(state, user, quser) == -1)
    return;
  if (query(state, "delete from `%s` where `%s` = '%s'",
	    state->quota_db_mysql_table,
	    state->quota_db_mysql_col_user,
	    quser) == -1)
    return;
}

char *quotadb_mysql_check(struct fstate *state, char *user) {
  quser_t quser;
  MYSQL_RES *res;
  MYSQL_ROW row;
  unsigned long date = 0;
  int num = 0;
  int code = 0;
  if (quote_user(state, user, quser) == -1)
    return NULL;
  if (query(state,
	    "select `%s`, `%s`, `%s` from `%s` where `%s` = '%s' limit 1",
	    state->quota_db_mysql_col_time,
	    state->quota_db_mysql_col_code,
	    state->quota_db_mysql_col_num,
	    state->quota_db_mysql_table,
	    state->quota_db_mysql_col_user,
	    quser) == -1)
    return NULL;
  if ( (res = mysql_use_result(&state->quota_db_mysql)) == NULL )
    return NULL;
  if ( (row = mysql_fetch_row(res)) == NULL ) {
    mysql_free_result(res);
    return NULL;
  }
  if (row[0]) date = strtoul(row[0], NULL, 10);
  if (row[1]) code = strtol(row[1], NULL, 10);
  if (row[2]) num  = strtol(row[2], NULL, 10);

  mysql_free_result(res);

  if (state->date - date < state->quota_db_timeout) {
    num ++;
    if (state->quota_maxchecks > 0 && num > state->quota_maxchecks)
      code = 552;
    query(state,
	  "update low_priority `%s`"
	  " set `%s` = %lu, `%s` = %d, `%s` = `%s` + 1"
	  " where `%s` = '%s'",
	  state->quota_db_mysql_table,
	  state->quota_db_mysql_col_time,
	  (unsigned long)state->date,
	  state->quota_db_mysql_col_code,
	  code,
	  state->quota_db_mysql_col_num,
	  state->quota_db_mysql_col_num,
	  state->quota_db_mysql_col_user,
	  quser
	  );
    
    return seterr(state, code, "4.2.2",
		  "<%s>: Mailbox full -- Quota exceeded", user);
    
  } else {
    query(state, "delete from `%s` where `%s` = '%s'",
	  state->quota_db_mysql_table,
	  state->quota_db_mysql_col_user,
	  quser);
  }
  return NULL;
}

void quotadb_mysql_add(struct fstate *state, char *user, int code) {
  quser_t quser;
  if (quote_user(state, user, quser) == -1)
    return;
  if (query(state, 
	    "REPLACE LOW_PRIORITY `%s` (`%s`,`%s`,`%s`,`%s`)"
	    " VALUES ('%s',%d,%d,%lu)",
	    state->quota_db_mysql_table,
	    state->quota_db_mysql_col_user,
	    state->quota_db_mysql_col_code,
	    state->quota_db_mysql_col_num,
	    state->quota_db_mysql_col_time,
	    quser, code, 1, (unsigned long)state->date) == -1)
    return;
  return;
}

/*
 *  quotadb_mysqladd_* functions
 */

int quotadb_mysqladd_open(struct fstate *state) {
  if (state->quota_db_mysql_col_user == NULL ||
      state->quota_db_mysql_table == NULL) {
    syslog(LOG_ERR, "quotadb_mysqladd: NULL user column or table name");
    return -1;
  }
  return quotadb_mysql_open_server(state);
}

void quotadb_mysqladd_add(struct fstate *state, char *user, int code) {
  quser_t quser;
  if (quote_user(state, user, quser) == -1)
    return;
  query(state,
	"REPLACE LOW_PRIORITY `%s` SET `%s`='%s'",
	state->quota_db_mysql_table,
	state->quota_db_mysql_col_user,
	quser);
  return;
}

#endif
